import javax.microedition.lcdui.*;
import java.util.*;

public class GameScreen 
	extends Canvas
	implements Runnable
{
	Random generator;
	
	Graphics buffer;
	
	Image myImage;
	Image bufferImage;
	
	int colour;
	int myImageX;
	int myImageY;
	
	int imageDirection;
	
	//Default constructor for our GameScreen class.
	public GameScreen()
	{
		//set up the games random number generator
		generator = new Random( System.currentTimeMillis() );
		
		//create all needed images
		createImages();
		
		//set image startup position to center of screen
		myImageX = getWidth()/2;
		myImageY = getHeight()/2;
		//create a new Thread on this Runnable and start it immediately
		new Thread( this ).start();		
	}
	
	/*
	 * Creates all the needed images, called upon creation
	 * of our GameScreen class 
	 */	
	public void createImages()
	{
		try
		{
			//if device doenst do automatic double buffering
			if( !isDoubleBuffered() )
			{
				//create offscreen Image
				bufferImage = Image.createImage( getWidth(), getHeight() );
				//get a Graphics context to we can render onto the bufferImage
				buffer = bufferImage.getGraphics();
			}
			myImage = Image.createImage("/sprite.png");
		}
		catch( Exception e )
		{
			e.printStackTrace();
		}
	}
	
	/*
	 * run() method defined in the Runnable interface, called by the 
	 * Virtual machine when a Thread is started.
	 */
	public void run()
	{
		while( true )
		{
			//set wanted loop delay to 15th of a second
			int loopDelay = 1000 / 15;
			//get the time at the start of the loop
			long loopStartTime = System.currentTimeMillis();
			//call our tick() fucntion which will be our games heartbeat
			tick();
			//get time at end of loop
			long loopEndTime  = System.currentTimeMillis();
			//caluclate the difference in time from start til end of loop
			int loopTime = (int)(loopEndTime - loopStartTime); 
			//if the difference is less than what we want
			if( loopTime < loopDelay )
			{
				try
				{
					//then sleep for the time needed to fullfill our wanted rate
					Thread.sleep( loopDelay - loopTime );
				}
				catch( Exception e )
				{
				}
				
			}
		}
	}
	
	/*
	 * Our games main loop, called at a fixed rate by our game Thread
	 */
	public void tick()
	{
		int myImageSpeed = 4;
		switch( imageDirection )
		{
			case LEFT:
			myImageX-=myImageSpeed;
			break;
			
			case RIGHT:
			myImageX+=myImageSpeed;
			break;
			
			case UP:
			myImageY-=myImageSpeed;
			break;
			
			case DOWN:
			myImageY+=myImageSpeed;
			break;
		}
		//schedule a repaint of the Canvas 
		repaint();
		//forces any pending repaints to be serviced, and blocks until 
		//paint() has returned
		serviceRepaints();
	}
	
	/* 
	 * called when the Canvas is to be painted
	 */
	protected void paint( Graphics g )
	{
		//cache a reference to the original Graphics context
		Graphics original = g;
		//if device doenst do automatic double buffering
		if( !isDoubleBuffered() )
		{
			//change the g object reference to the back buffer Graphics context
			g = buffer;
		}
		//set the current color of the Graphics contect to the specified RRGGBB colour
		g.setColor( colour );
		//draw a filled rectangle at x,y coordinates 0, 0 with a width
		// and height equal to that of the Canvas itself
		g.fillRect( 0, 0, this.getWidth(), this.getHeight() );
		
		//draw an image to the screen
		g.drawImage( myImage, myImageX, myImageY, Graphics.VCENTER | Graphics.HCENTER );
		
		if( !isDoubleBuffered() )
		{
			//draw the offscreen Image to the original graphics context 
			original.drawImage( bufferImage, 0, 0, Graphics.TOP | Graphics.LEFT );
		}		
	}
	
	/* 
	* called when a key is pressed and this Canvas is the
	* current Displayable 
	*/
	protected void keyPressed( int keyCode )
	{
		//get the game action from the passed keyCode
		int gameAction = getGameAction( keyCode );
		switch( gameAction )
		{
			case LEFT:
			//move image left
			imageDirection = LEFT; 
			break;
			
			case RIGHT:
			//move image right
			imageDirection = RIGHT;
			break;
			
			case UP:
			//move image up
			imageDirection = UP;
			break;
			
			case DOWN:
			//move image down
			imageDirection = DOWN;
			break;
			
			case FIRE:
			//set current to a random colour
			colour = generator.nextInt()&0xFFFFFF;
			break;
		}
	}

	/* 
	* called when a key is released  and this Canvas is the
	* current Displayable 
	*/
	protected void keyReleased( int keyCode )
	{
		imageDirection = 0;
	}
}
